package de.communardo.confluence.plugins.addlink;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.opensymphony.webwork.interceptor.ParameterAware;

import de.communardo.confluence.plugins.configuration.ConfigurationAction;
import de.communardo.confluence.plugins.configuration.StaticLinkHelper;

/**
 * 
 * @author Bill Martin
 * 
 */

public class AddLinkAction extends ConfigurationAction implements ParameterAware {

    private static final long serialVersionUID = -3331518468289780221L;
    private Map parameters;

    public String execute() throws Exception {
        StaticLinkHelper.storeLinks(getLinks());
        setSuccess(true);
        return SUCCESS;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void validate() {
        setUp();
        super.validate();
        int i = 0;
        for (Link link : getLinks()) {
            if (link.getAlias() != null && link.getAlias().trim().length() == 0) {
                addFieldError("alias_" + i, getText("addlink.error.alias.empty"));
            }
            if (link.getLink() != null && link.getLink().trim().length() == 0) {
                addFieldError("adress_" + i, getText("addlink.error.adress.empty"));
            } else if (!link.getLink().matches("^[^:/]+://.*"))
                addFieldError("adress_" + i, getText("addlink.error.adress.protocol"));
            i++;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Link> getLinks() {
        if (super.getLinks() == null) {
            List<Link> links = new ArrayList<Link>();
            String[] field = (String[]) this.parameters.get("numberOfRows");
            int numbersOfRow = new Integer(field[0]).intValue();
            for (int i = 0; i < numbersOfRow; i++) {
                String[] alias = (String[]) this.parameters.get("newalias_" + i);
                String[] address = (String[]) this.parameters.get("newadress_" + i);
                links.add(new Link(alias[0], address[0]));
            }
            super.setLinks(links);
        }
        return super.getLinks();
    }

    /**
     * 
     * @param fieldName
     * @return errors for a given field
     */
    public Collection<String> getFieldErrors(String fieldName) {
        Collection<String> errors = (Collection<String>) getFieldErrors().get(fieldName);
        if (errors == null) {
            errors = new ArrayList<String>();
        }
        return errors;
    }

    public Map getParameters() {
        return parameters;
    }

    public void setParameters(Map parameters) {
        this.parameters = parameters;
    }
}
