package de.communardo.confluence.plugins.createsubspace;

import java.util.List;

import org.apache.log4j.Logger;

import com.atlassian.confluence.security.SpacePermission;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.spaces.actions.AbstractSpaceAction;
import com.atlassian.confluence.util.GeneralUtil;

/**
 * 
 * @author Bill Martin
 * 
 */
public class ChangeParentSpaceAction extends AbstractSpaceAction {

    /**
	 * 
	 */
    private static final long serialVersionUID = -7381066909888180812L;
    private String parentKey = null;
    private Space parentSpace = null;
    private static final Logger LOGGER = Logger.getLogger(ChangeParentSpaceAction.class);
    protected String key;
    protected Space space;
    protected Space originalSpace;
    protected SpaceManager spaceManager;

    /**
     * Method change the parent space. If the name of the parentspace is "dashboard" it will be a
     * "Top-Level-Space".
     * 
     * @overrides com.opensymphony.xwork.ActionSupport.execute()
     * @throws Exception
     * @return "SUCCESS" Return "SUCCESS" if all is correct.
     */
    @Override
    public String execute() throws Exception {
        String res = super.execute();
        if (res.equals(SUCCESS)) {
            if (this.parentKey.equalsIgnoreCase(CreateSpaceHierarchyAction.SPACE_KEY_DASHBOARD)) {
                Space parentSpace = new Space();
                parentSpace.setKey("");
                CreateSubSpaceUtils.assignParentSpace(parentSpace, this.getSpace());
            } else {
                CreateSubSpaceUtils.assignParentSpace(this.getParentSpace(), this.getSpace());
            }
        }
        return SUCCESS;
    }

    @Override
    public String doDefault() throws Exception {
        return super.doDefault();
    }

    public String getOldParentKey() {
        String spcKey = null;
        Space spc = this.getSpace();
        if (spc != null) {
            Space parentSpace = CreateSubSpaceUtils.getParentSpace(spc);
            if (parentSpace != null) {
                spcKey = parentSpace.getKey();
            }
        }
        return spcKey;
    }

    public Space getParentSpace() {
        if (this.parentSpace == null) {
            this.parentSpace = spaceManager.getSpace(this.getParentKey());
        }
        return this.parentSpace;
    }

    @Override
    public void validate() {
        // If the parent key is "dashboard" it won't be validated.
        if (!this.parentKey.equalsIgnoreCase(CreateSpaceHierarchyAction.SPACE_KEY_DASHBOARD)) {
            if (this.getSpace() == null) {
                addFieldError("parentKey",
                        getText(GeneralUtil.getI18n().getText("change.parentspace.action.java.null")));
            } else {
                if (spaceManager.getSpace(this.getParentKey()) == null) {
                    addFieldError("parentKey",
                            getText(GeneralUtil.getI18n().getText("change.parentspace.action.java.null2")));

                } else if (CreateSubSpaceUtils.hasCyclicReference(this.getParentSpace(), this.getSpace())) {
                    addFieldError("parentKey",
                            getText(GeneralUtil.getI18n().getText("change.parentspace.action.java.cycle")));
                }
            }
            super.validate();
        }
    }

    public void setParentKey(String pK) {
        this.parentKey = pK;
    }

    public String getParentKey() {
        return this.parentKey;
    }

    /**
     * Adds space administration permission to required permission set.
     * 
     * @return the set of required permissions to change the parent space
     */
    @Override
    protected List getPermissionTypes() {
        List permissionTypes = super.getPermissionTypes();
        if (getSpace() != null) {
            addPermissionTypeTo(SpacePermission.ADMINISTER_SPACE_PERMISSION, permissionTypes);
        }
        return permissionTypes;
    }

    public String getKey() {
        return key;
    }

    public String getSpaceKey() {
        return getKey();
    }

    public void setKey(String key) {
        this.key = key.trim();
        space = null;
    }

    public boolean isValidSpaceKey() {
        return Space.isValidSpaceKey(key);
    }

    /**
     * Used only by unit tests
     */
    public void setSpace(Space space) {
        this.space = space;
    }

    public Space getSpace() {
        if (space == null && isValidSpaceKey()) {
            space = spaceManager.getSpace(key);
            if (space != null) {
                try {
                    originalSpace = (Space) space.clone();
                } catch (CloneNotSupportedException e) {
                    LOGGER.error("Could not clone space?" + e, e);
                }
            }
        }

        return space;
    }

    public void setSpaceManager(SpaceManager spaceManager) {
        this.spaceManager = spaceManager;
    }
}