package de.communardo.confluence.plugins.createsubspace;

import org.andya.confluence.plugins.metadata.space.SpaceUtils;
import org.andya.confluence.utils.ContentService;

import com.atlassian.confluence.core.ContentPropertyManager;
import com.atlassian.confluence.core.FormatSettingsManager;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.spring.container.ContainerManager;
import com.atlassian.user.UserManager;

/**
 * 
 * @author Bill Martin
 * 
 */
public class CreateSubSpaceUtils {

    private static ContentService contentService = null;

    /**
     * Needed for compliance with metadata plugin.
     * 
     * @see org.andya.confluence.plugins.metadata.AbstractMetadataMacro#getContentService()
     * @return a wrapper for multiple manager objects
     */
    private static final ContentService getContentService() {
        if (contentService == null) {
            contentService = new ContentService(getContentPropertyManager(), getSpaceManager(),
                    getUserManager(), getFormatSettingsManager());
        }
        return contentService;
    }

    /**
     * uses ContainerManager to get format settings manager
     * 
     * @return the format settings manager
     */
    private static final FormatSettingsManager getFormatSettingsManager() {
        return (FormatSettingsManager) ContainerManager.getComponent("formatSettingsManager");
    }

    /**
     * uses ContainerManager to get user manager
     * 
     * @return the user manager
     */
    private static final UserManager getUserManager() {
        return (UserManager) ContainerManager.getComponent("userManager");
    }

    /**
     * uses ContainerManager to get content property manager
     * 
     * @return the content property manager
     */
    private static final ContentPropertyManager getContentPropertyManager() {
        return (ContentPropertyManager) ContainerManager.getComponent("contentPropertyManager");
    }

    /**
     * uses ContainerManager to get space manager
     * 
     * @return the space manager
     */
    private static final SpaceManager getSpaceManager() {
        return (SpaceManager) ContainerManager.getComponent("spaceManager");
    }

    /**
     * Assigns parent space to another space.
     * 
     * @param parentSpace
     *            the parent space
     * @param childSpace
     *            the space to which the parent space will be assigned
     * @see org.andya.confluence.plugins.metadata.space.ParentSpaceMacro
     * @return true if storing succeeded
     */
    public static boolean assignParentSpace(Space parentSpace, Space childSpace) {
        // store space key in content properties, analog to ParentSpaceMacro
        if (parentSpace != null) {
            SpaceUtils.setParentSpace(getContentService(), childSpace, parentSpace);
        }
        return true;

    }

    /**
     * Compare two spaces for equality. Spaces are equal if their keys are equal.
     * 
     * @param space_1
     *            one space
     * @param space_2
     *            another space
     * 
     * @return true if spaces are equal
     */
    public static boolean spaceEquals(Space space_1, Space space_2) {
        return space_1.getKey().equals(space_2.getKey());
    }

    /**
     * Retrieves parent space for given space.
     * 
     * @param spc
     *            the space for which to retrieve the parent
     * @return the parent space (null if there is no parent space)
     */
    public static Space getParentSpace(Space spc) {
        Space parent = null;
		try{
			parent = SpaceUtils.getParentSpace(getContentService(), spc);
		}
		catch(Exception e){
		}
		return parent;
    }

    /**
     * Checks whether given parent space is amongst children of given child space.
     * 
     * @param parentSpace
     *            the parent space
     * @param childSpace
     *            the child space
     * @return true if cyclic reference exists
     */
    public static boolean hasCyclicReference(Space parentSpace, Space childSpace) {
        boolean isCyclic = false;
        while ((!isCyclic) && parentSpace != null) {
            if (spaceEquals(parentSpace, childSpace)) {
                isCyclic = true;
            }
            parentSpace = getParentSpace(parentSpace);
        }
        return isCyclic;
    }
}
