package de.communardo.confluence.plugins.displaysubspace;

import java.util.List;

public class NodeHelper {

    private final String activeSpace;

    private final List permittedSpaced;

    public NodeHelper(String activeSpace, List permittedSpaces) {
        this.activeSpace = activeSpace;
        this.permittedSpaced = permittedSpaces;
    }

    /**
     * Check if a tree node is visible for the given user.
     * 
     * @param node
     *            the given tree node
     * @param user
     *            the given user
     * @return true if at least one child of the given tree node is visible
     */
    public boolean isVisible(TreeNode node) {

        boolean visible = this.permittedSpaced.contains(node.getSpace());
        return visible;
    }

    /**
     * returns true if the given node represents the current shown space
     * 
     * @param node
     * @return
     */
    public boolean isActive(TreeNode node) {

        return (this.activeSpace != null && node.getSpace().getKey().equals(this.activeSpace));
    }

    /**
     * returns true if one of the node childs represents the current shown space
     * 
     * @param node
     * @return
     */
    public boolean hasChildInContext(TreeNode node) {

        boolean result = false;
        for (TreeNode child : node.getChildren()) {
            if (this.isActive(child)) {
                result = true;
            } else {
                result = this.hasChildInContext(child);
            }
            if (result == true) {
                return true;
            }
        }
        return false;
    }
}
