package de.communardo.confluence.plugins.displaysubspace;

import java.util.List;
import java.util.Map;

import com.atlassian.confluence.renderer.radeox.macros.MacroUtils;
import com.atlassian.confluence.security.Permission;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.util.velocity.VelocityUtils;

import de.communardo.confluence.plugins.util.ComponentManager;

/**
 * Render a TreeNode.
 * 
 * @author Bill Martin
 * 
 */
public class TreeNodeRenderer {

    private final String velocityTemplatePath;
    private final Map<String, Object> contextMap;

    /**
     * 
     * @param hierarchyBuilder
     *            The HierarchyBuilder
     */
    public TreeNodeRenderer(Map<String, Object> context, String velocityTemplatePath,
            String currentSpace, List permittedSpaces) {
        this.velocityTemplatePath = velocityTemplatePath;

        if (context == null) {
            contextMap = MacroUtils.defaultVelocityContext();
        } else {
            contextMap = context;
        }
        contextMap.put("user", AuthenticatedUserThreadLocal.getUser());
        contextMap.put("treeNodeRenderer", this);
        contextMap.put("labelManager", ComponentManager.getLabelManager());
        contextMap.put("permissionManager", ComponentManager.getPermissionManager());
        contextMap.put("view", Permission.VIEW);
        contextMap.put("nodeHelper", new NodeHelper(currentSpace, permittedSpaces));
    }

    public String renderTreeNode(TreeNode node) {
        this.contextMap.put("node", node);
        return VelocityUtils.getRenderedTemplate(this.velocityTemplatePath, contextMap);
    }
}